<?php
/**
 * A class for reading text files
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FrameworkTextFileInputPointer extends BaseFileIo implements InputPointer {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Create a file pointer based on the given file. The file will also
	 * be verified here.
	 *
	 * @param	$fileName	The file name we shall pass to fopen()
	 * @throws	FileIsEmptyException		If the provided file name is empty.
	 * @throws	FileIoException				If the file is not reachable
	 * @throws	FileReadProtectedException	If the file cannot be read from
	 * @return	void
	 */
	public static final function createFrameworkTextFileInputPointer ($fileName) {
		// Some pre-sanity checks...
		if ((is_null($fileName)) || (empty($fileName))) {
			// No filename given
			throw new FileIsEmptyException(NULL, self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
		} elseif (!BaseFrameworkSystem::isReachableFilePath($fileName)) {
			// File does not exist!
			throw new FileIoException($fileName, self::EXCEPTION_FILE_NOT_REACHABLE);
		} elseif (!BaseFrameworkSystem::isReadableFile($fileName)) {
			// File does not exist!
			throw new FileReadProtectedException($fileName, self::EXCEPTION_FILE_CANNOT_BE_READ);
		}

		// Try to open a handler
		$filePointer = fopen($fileName, 'r');
		if ((is_null($filePointer)) || ($filePointer === FALSE)) {
			// Something bad happend
			throw new FileIoException($fileName, self::EXCEPTION_FILE_POINTER_INVALID);
		} // END - if

		// Create new instance
		$pointerInstance = new FrameworkTextFileInputPointer();

		// Set file pointer and file name
		$pointerInstance->setPointer($filePointer);
		$pointerInstance->setFileName($fileName);

		// Return the instance
		return $pointerInstance;
	}

	/**
	 * Read data a file pointer
	 *
	 * @return	$data	Read data from file
	 */
	public function readFromFile () {
		// Read 1024 Byte data from the file pointer and return it
		return $this->read(1024);
	}

	/**
	 * Reads a line, maximum 4096 Bytes from current file pointer
	 *
	 * @return	$data	Read data from file
	 */
	public function readLine () {
		// Read whole line from the file pointer and return it
		return $this->read();
	}

	/**
	 * Reads given amount of bytes from file.
	 *
	 * @param	$bytes	Amount of bytes to read or whole line (only text files)
	 * @return	$data	Data read from file
	 * @throws	NullPointerException	If the file pointer instance
	 *									is not set by setPointer()
	 * @throws	InvalidResourceException	If there is being set
	 */
	public function read ($bytes = NULL) {
		// Some sanity checks
		if (is_null($this->getPointer())) {
			// Pointer not initialized
			throw new NullPointerException($this, self::EXCEPTION_IS_NULL_POINTER);
		} elseif (!is_resource($this->getPointer())) {
			// Pointer is not a valid resource!
			throw new InvalidResourceException($this, self::EXCEPTION_INVALID_RESOURCE);
		}

		// Is $bytes set?
		if (is_int($bytes)) {
			// Try to read given characters
			$data = fgets($this->getPointer(), $bytes);
		} else {
			// Try to read whole line
			$data = fgets($this->getPointer());
		}

		// Then return it
		return $data;
	}

	/**
	 * Analyzes entries in index file. This will count all found (and valid)
	 * entries, mark invalid as damaged and count gaps ("fragmentation"). If
	 * only gaps are found, the file is considered as "virgin" (no entries).
	 *
	 * @return	void
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function analyzeFile () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

	/**
	 * Advances to next "block" of bytes
	 *
	 * @return	void
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function next () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

	/**
	 * Checks wether the current entry is valid (not at the end of the file).
	 * This method will return TRUE if an emptied (nulled) entry has been found.
	 *
	 * @return	$isValid	Whether the next entry is valid
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function valid () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

	/**
	 * Gets current seek position ("key").
	 *
	 * @return	$key	Current key in iteration
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function key () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}
}

// [EOF]
?>
